@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2018 A S Lewis
+# Copyright (C) 2011-2019 A S Lewis
 #
 # This program is free software: you can redistribute it and/or modify it under the terms of the GNU
 # General Public License as published by the Free Software Foundation, either version 3 of the
@@ -824,7 +824,7 @@
             $winObj->winType,
             $winObj->winName,
             $winObj->winWidget,
-            $winObj->wnckWin,
+            undef,
             $winObj->owner,
             $winObj->session,
         );
